#!/usr/bin/perl
#version 1.0.0.11p

use strict;
use LWP::UserAgent;

#your cacti script directory
#my $SCRIPTPATH='C:/XAMPP/htdocs/cacti/scripts/';
my $SCRIPTPATH='/var/www/html/scripts/';

#BASIC AUTHENTICATION
my $USER='user';
my $PASS='eds';

#family of sensors to look for

#hex family code 10=ds1920,ds1820,ds18s20 CSV of family codes to assume are temp probes
my @TFAMILY=('.*10$|.*28$|.*26$');
my @HFAMILY=('.*12$');

#hex family code 12=ds2406,ds2407 CSV of family codes to assume are humidity sensors

#list all temperature codes first followed by humidity codes (its important!)
my @FAMILY=('10','28','26','12');

#return  values in  fahrenheit?
my $FAHRENHEIT=1;

#port that web server is running on (usually 80)
my $PORT=80;

my $agent='Cacti HA7Net script';
#####################################################################################
#no user servicable parts past here
#####################################################################################
my $server= $ARGV[0] . ':' .$PORT;
my @ADDRESSES;
my @HADDRESSES;
my @TADDRESSES;
my $UA = new LWP::UserAgent($agent);
my $REQ = HTTP::Headers->new;
my $CONTENT;

if ($ARGV[0] =~/\b25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?\.25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?\.25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?\.25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?\b/){
        if (($ARGV[1] eq 'index')||($ARGV[1] eq 'num_indexes')){
                search_addresses();
                if ($ARGV[1] eq 'index'){
                        foreach (@ADDRESSES){
                                print "$_\n";
                        }
                        exit;
                }elsif($ARGV[1] eq 'num_indexes'){
                        my $count= @ADDRESSES;
                        print "$count\n";
                        exit;
                }else{
                        usage();
                }
        }elsif($ARGV[1] eq 'get'){
                if ($ARGV[2] =~/[0-9A-F]{16}/){
                        split_addresses($ARGV[2]);
                        get_reading();
                }else{
                        usage();
                }

        }elsif($ARGV[1] eq 'query'){
                if ($ARGV[2] eq 'address'){
                        search_addresses();
                        foreach (@ADDRESSES){
                                print "$_:$_\n";
                        }
                        exit;
                }elsif ($ARGV[2] eq 'name'){
                        search_addresses();
                        my $name;
                        foreach (@ADDRESSES){
                                print "$_:";
                                $name=findSensorName($_);
                                print "$name\n";
                        }
                        exit;
                }elsif ($ARGV[2] eq 'type'){
                        search_addresses();
                        findSensorType();
                        exit;
                }else{
                        usage();
                }
        }else{
                usage();
        }
}else{
        usage();
}

sub get_reading{
        if (@TADDRESSES){
                $REQ =HTTP::Request->new(GET => "http://$server/1Wire/ReadTemperature.html?Address_Array=@TADDRESSES");
                $REQ->authorization_basic($USER, $PASS);
                $CONTENT = $UA->request($REQ)->as_string;
                for (my $i;$CONTENT =~ /<td><INPUT CLASS="HA7Value" NAME="Temperature_\d+".*?VALUE="([0-9\.]+)">/g;$i++){
                        my $temp=$1;
                        if ($FAHRENHEIT){
                                $temp=($temp*1.8)+32;
                        }
                        #print "@TADDRESSES[$i]:$temp\n";
                        print "$temp\n";
                }
        }
        if (@HADDRESSES){
                $REQ =HTTP::Request->new(GET => "http://$server/1Wire/ReadAnalogProbe.html?Address_Array=@HADDRESSES");
                $REQ->authorization_basic($USER, $PASS);
                $CONTENT = $UA->request($REQ)->as_string;
                for (my $i=0;$CONTENT =~ /<td><INPUT CLASS="HA7Value" NAME="Probe_Value_\d+".*?VALUE="([0-9\.]+)">/g;$i++){
                        #print "@HADDRESSES[$i]:$1\n";
                        print "$1\n";
                }
        }
}

sub search_addresses{
        $REQ = HTTP::Request->new(GET => "http://$server/1Wire/Search.html");
        $REQ->authorization_basic($USER, $PASS);
        $CONTENT = $UA->request($REQ)->as_string;
        foreach (@FAMILY){
                for (;$CONTENT =~ /<tr><td><INPUT CLASS="HA7Value" NAME="Address_\d+" ID="ADDRESS_\d+" TYPE="text" VALUE="(.*?$_)">/g;){
                        my $address=$1;
                        push (@ADDRESSES ,$address);
                        split_addresses($address);
                }
        }
}

sub split_addresses{
        my ($in) = @_;
        if ($in =~/@TFAMILY/){
                push (@TADDRESSES, $in);
        }elsif($in =~/@HFAMILY/){
                push (@HADDRESSES, $in);
        }
}

sub findSensorName {
        my($name) = @_;
        my ($add,$nam);
        my $namefile ='sensors.txt';
        open(LOOKUP, $SCRIPTPATH . $namefile) || die "could not locate $SCRIPTPATH" . "sensors.txt";
        while(<LOOKUP>) {
                chomp;
                s/^\s+//;       #remove leading spaces
                s/#.*//;        #remove commented lines
                ($add,$nam)=split/\t/;

                if ($name eq $add){
                        $name = $nam;
                }
        }
                close(LOOKUP);
                return $name;
}

sub findSensorType {
        foreach (@TADDRESSES){
                print "$_:Temperature\n";
        }
        foreach (@HADDRESSES){
                print "$_:Humidity\n";
        }
}

sub usage {
        print "usage:\n./ha7_get.pl {ip} index\n\tReturns the addresses of all connected sensors\n";
	print "./ha7_get.pl {ip} num_indexes\n\t Returns the number of connected humidity and temperature sensors\n";
	print "./ha7_get.pl {ip} query (name|address|type)\n\tReturns all names (assigned in sensors.txt),\n\tall addresses (same as the index)\n\tor the type (humidity or temperture)\n";
        print "./ha7_get.pl {ip} get {address}\n\tReturns the reading for a single sensor\n\n";
}